@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 # -------------------------------------------------------
-# $Header: /home/cvs/dobackup/dobackup.pl,v 4.18 2002/07/09 08:55:31 imorgan Exp $
+# $Header: /home/cvs/dobackup/dobackup.pl,v 4.22 2002/12/03 17:24:37 imorgan Exp $
 # -------------------------------------------------------
 #
 # Copyright 1997-2002 Webcon, Inc.
@@ -24,7 +24,7 @@
 require v5.6.0;
 
 # External Calls:
-use IniConf;
+use Config::IniFiles;
 use strict;
 use warnings;
 use Getopt::Long; # For easy command line parsing
@@ -66,7 +66,9 @@ our($Year);
 our(@HostsInConfig);
 our(@HostsToProcess);
 our(%IP);
+our(%IncrMode);
 our(%Timestamp);
+our(%TimestampFull);
 our(%Mode);
 our(%Destination);
 our(%MediaUnit);
@@ -203,7 +205,7 @@ sub ShowSyntax {
   my($temp)=('');
 
   print "\nUsage: $ExecutableName <options>\n";
-  $temp = '$Revision: 4.18 $';
+  $temp = '$Revision: 4.22 $';
   ($temp) = ( $temp =~ /^\$Revision\:\ (.*?)\ \$$/ );
   print 'Version: ',"$temp\n";
   print <<EOF;
@@ -540,7 +542,11 @@ sub PrepareBackup{
     if ( $TimeStampOverride && $TimeStampOverride > 0 ) {
       $StampCTime=$TimeStampOverride;
     } else {
-      $StampCTime=$Timestamp{$Host};
+      if ( $IncrMode{$Host} eq 'Timewarp' ) {
+        $StampCTime=$TimestampFull{$Host};
+      } else {
+        $StampCTime=$Timestamp{$Host};
+      }
     }
 
     $Debug && print "Building for incremental backup from $StampCTime.\n";
@@ -700,15 +706,15 @@ LOOP2:    foreach (@NonDirs) {
         }
       }
 
-      # If it is a broken link it will have an empty CTime. This always backs up broken links!
-      if ( (! $TempCTime) && (-l "$_") ) { 
-        print BACKTEMP "$_\n"; 
-        next; 
-        ($Debug > 1) && print "Adding $_\n";
+      # If it is a broken link it will have an empty CTime, so stat the link itself.
+      if ( (! $TempCTime) && (-l "$_") ) {
+        (undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,$TempCTime) = lstat("$_");
+        ($Debug > 2) && print "Checking broken link: $_\n";
       }
 
+      # Would this _ever_ happen?!
       if ( ! $TempCTime ) { # Always backup files non-links with Empty CTimes
-        print "TempCTTime Empty for $_\n"; 
+        print "TempCTTime Empty for $_\n";
         print BACKTEMP "$_\n";
         ($Debug > 1) && print "Adding $_\n";
         next;
@@ -801,7 +807,11 @@ sub PerformBackup{
     if ( $TimeStampOverride && $TimeStampOverride > 0 ) {
       $Stamp=$TimeStampOverride;
     } else {
-      $Stamp=$Timestamp{$Host};
+      if ( $IncrMode{$Host} eq 'Timewarp' ) {
+        $Stamp=$TimestampFull{$Host};
+      } else {
+        $Stamp=$Timestamp{$Host};
+      }
     }
     $Debug && print ("Performing Incremental Backup from epoc+$Stamp!\n");
 
@@ -866,6 +876,12 @@ sub PerformBackup{
     #Filenames in Disk and Filesystem mode must be adapted for uniqueness
 
     $Filename=$Host."_".$BackupDay;
+    if ($opt_fullbackup || $Timestamp{$Host} == 0) {
+      $Filename .= ".full";
+    } else {
+      $Filename .= ".incr" if ($IncrMode{$Host} ne 'Timewarp');
+      $Filename .= ".warp" if ($IncrMode{$Host} eq 'Timewarp');
+    }
     $Filename .= ".debug" if ($Debug);
 
     $Debug && print "Command: $Umask 0277; $Cat $ToBackupTempFile | $BackupProgDebug $Destination{$Host}/$Filename 2>> $BackupErrorFile.$Host";
@@ -875,14 +891,14 @@ sub PerformBackup{
 
   if( $Return == 0 ) {} # no problem
   elsif ($Return == 0xff00 ) {
-     print("ERROR: cat ($Cat) to backup command ($Debug?$BackupProgDebug:$BackupProg) failed!\n\nAborting backup!\n");
+     print("ERROR: cat ($Cat) to backup command (",$Debug?$BackupProgDebug:$BackupProg,") failed!\n\nAborting backup!\n");
      exit 1;
   } elsif ($Return > 0x80) {
      $Return >>= 8;
-     print("ERROR: cat ($Cat) to backup command ($Debug?$BackupProgDebug:$BackupProg) ran with non-zero exit status $Return\n\nAborting backup!\n");
+     print("ERROR: cat ($Cat) to backup command (",$Debug?$BackupProgDebug:$BackupProg,") ran with non-zero exit status $Return\n\nAborting backup!\n");
      exit 1;
   } else {
-    print "ERROR: cat ($Cat) to backup command ($Debug?$BackupProgDebug:$BackupProg) ran with";
+    print "ERROR: cat ($Cat) to backup command (",$Debug?$BackupProgDebug:$BackupProg,") ran with";
     if ($Return & 0x80){
       $Return &= ~0x80;
       print "coredump from ";
@@ -1002,23 +1018,31 @@ sub Cleanup{
   open (MAILTHIS, "|$MailProg $BackupAdministrator");
   print MAILTHIS "To: $BackupAdministrator\nSubject: $Host ",($Debug?"Debug-mode":"Automated")," Backup Summary\n\n";
 
-  $temp = '$Revision: 4.18 $';
+  $temp = '$Revision: 4.22 $';
   ($temp) = ( $temp =~ /^\$Revision\:\ (.*?)\ \$$/ );
 
   print MAILTHIS "This is an automated backup report from $ExecutableName v$temp\n\n";
 
   $Debug && print MAILTHIS "Note:            This backup was performed in debug mode!\n";
+  ($IncrMode{$Host} eq 'Timewarp') && print MAILTHIS "Note:            This backup was performed in Timewarp mode!\n";
 
   if ( ($Mode{$Host} eq 'Disk') || ($Mode{$Host} eq 'Filesystem') ) {
     $temp = $Destination{$Host}.'/'.$Host.'_'.$BackupDay;
+    if ($opt_fullbackup || $Timestamp{$Host} == 0) {
+      $temp .= ".full";
+    } else {
+      $temp .= ".incr" if ($IncrMode{$Host} ne 'Timewarp');
+      $temp .= ".warp" if ($IncrMode{$Host} eq 'Timewarp');
+    }
     $temp .= ".debug" if ($Debug);
+
     print MAILTHIS "File:            $temp\n";
     $temp = `$LS -s -h $temp`;
     ($temp) = ( $temp =~ /^\s*(.*?)\s.*$/ );
     $temp =~ s/\b(\d+)\w*[kK]\b/$1 KiB/g;
     $temp =~ s/\b(\d+)\w*M\b/$1 MiB/g;
     $temp =~ s/\b(\d+)\w*G\b/$1 GiB/g;
-        
+
     print MAILTHIS "Size:            $temp\n";
     $Debug && print "Size: $temp\n";
   }
@@ -1064,7 +1088,7 @@ sub Cleanup{
 # Syntax: &LoadConfig
 sub LoadConfig {
   my($HostName,$temp,@temphosts);
-  $IniConfig = IniConf->new( -file => $ConfigFile );
+  $IniConfig = Config::IniFiles->new( -file => $ConfigFile );
   if (!defined($IniConfig)) {
     if ( defined(@IniConf::Errors) ) {
       die("Could not read $ConfigFile:\n",@IniConf::Errors,"\n");
@@ -1113,7 +1137,17 @@ sub LoadConfig {
     }
 
     $IP{$HostName}          = $IniConfig->val($HostName,'IP');
+    $IncrMode{$HostName}    = $IniConfig->val($HostName,'IncrementalMode');
+    # Backward-compat: if missing, assume normal.
+    if ( ! defined($IncrMode{$HostName}) ) {
+      $IncrMode{$HostName}  = 'Normal';
+    }
+
     $Timestamp{$HostName}   = $IniConfig->val($HostName,'Timestamp');
+    $TimestampFull{$HostName}   = $IniConfig->val($HostName,'TimestampFull');
+    if ( ! defined($TimestampFull{$HostName}) ) {
+      $TimestampFull{$HostName} = $Timestamp{$HostName};
+    }
 
     $temp= $IniConfig->val($HostName,'MediaSet');
     $MediaSet{$HostName}    = $temp;
@@ -1141,7 +1175,9 @@ sub LoadConfig {
     $Debug && print "[$HostName]\n";
     $Debug && print "Auto=",$Auto{$HostName},"\n";
     $Debug && print "IP=",$IP{$HostName},"\n";
+    $Debug && print "IncrMode=",$IncrMode{$HostName},"\n";
     $Debug && print "Timestamp=",$Timestamp{$HostName},"\n";
+    $Debug && print "TimestampFull=",$TimestampFull{$HostName},"\n";
     $Debug && print "Mode=",$Mode{$HostName},"\n";
     $Debug && print "Destination=",$Destination{$HostName},"\n";
     $Debug && print "Paths=<<EOP\n";
@@ -1162,7 +1198,7 @@ sub LoadConfig {
 # UpdateConfig -- subroutine
 # ------------------------------
 # Description: Updates the Backup timestamps in the config file.
-# Calls: IniConf->new,Sections,setval
+# Calls: IniConf->setval,WriteConfig
 # Global Vars/Files Created:
 # Global Vars/Files Changed:
 # Global Reads:
@@ -1170,6 +1206,9 @@ sub LoadConfig {
 # Syntax: &UpdateConfig
 sub UpdateConfig {
   $IniConfig->setval($Host,'Timestamp',$StartTime);
+  if ( $opt_fullbackup || $Timestamp{$Host} == 0 ) {
+    $IniConfig->setval($Host,'TimestampFull',$StartTime);
+  }
   $IniConfig->WriteConfig($ConfigFile);
   $Debug && print "Config file backup timestamp updated for host $Host.\n";
 }
